import java
import herschel
from herschel.pacs.spg.spec.beams import ExtendedSource
from math import *
from herschel.ia.gui.plot.renderer.PlotLegendEngine import Position

###################################################################
#Classes and functions for the E2P packages
###################################################################

#Gaussian Model definition (Circular Gaussian with offset wrt centre of central spaxel)
class MyGaussSource(ExtendedSource):
    sigma = 0.0
    offRa = 0.0
    offDec = 0.0
    def __init__(self,diameter, offRa, offDec):
        self.sigma = diameter/2.3548
        self.offRa = offRa
        self.offDec = offDec
       
    def getFlux(self,relRa,relDec):
        flux = exp(-(0.5*((relRa-self.offRa)/self.sigma)**2+0.5*((relDec-self.offDec)/self.sigma)**2))
        return flux

    def renderGauss(self, size=109, dpix=0.5):
        self.image=Double2d(size, size)
        for y in range( size ):
           for x in range( size ):
             xim = (-dpix)*(x-(size/2+0.5))
             yim = (dpix)*(y-(size/2+0.5))
             self.image.set(y,x, source.getFlux(xim, yim))
        return self.image

    def getSourceModelImage(self, size=109, dpix=0.5):
        return self.renderGauss(size, dpix)

def flagSet(band,gd,flag,wve):
    flg=flag.copy()
    if (gd==1):
        if (band == "B3A"):
            sey=[55.0,70.0]
        elif (band == "B2A"):
            sey=[55.0,73.0]
        elif (band == "B2B"):
            sey=[70.0,95.0]
        else:
            sey=[102.,190.]
        idx=wve.where(wve < sey[0])
        flg[idx]= 1.0
        idx=wve.where(wve > sey[1])
        flg[idx]= 1.0
    else:
        sey=ABS(len(gd)/2.) - len(gd)/2
        if ((len(gd) > 0) &(sey != 0.0)):
            print "Error: you have an odd number of ""good"" entries"
            return
        flg[:]= 1.0
        for i in range(0,len(gd),2):
            idx=wve.where((wve >= gd[i])&(wve <= gd[i+1]))
            if (idx.length() == 0):
                print "Error: your limits",gd[i],gd[i+1],"are outside the range of the spectrum; start again"
                flg=None
            else: 
                flg[idx]= 0.0
    return(flg)

def colorShade(col=java.awt.Color(128,128,128), npts=9):
    r=col.getRed()
    g=col.getGreen()
    b=col.getBlue()
    nr=(npts-1)/2
    steps=Double1d.range(nr)*(1./(nr+1))+(1./(nr+1))
    listShades= [java.awt.Color(int(r*step), int(g*step), int(b*step)) for step in steps]
    listTints= [java.awt.Color(int(r+(step*(255-r))), int(g+(step*(255-g))), int(b+(step*(255-b)))) for step in steps]
    listAll=[listShades, [col], listTints]
    listFinal=[num for elem in listAll for num in elem]
    return listFinal

#A utility
def ls(directory="./",pattern="",case=1,verbose=1):
	"""ls(directory="./",pattern="",case=1)
Returns the files in "directory" (relative path allowed) with filename containing "pattern" (incl. wildcards)
case	: 0/1 for case (in)sensitive listing (prefer case = 1 for pattern filtering)
"""
	import sys,os,fnmatch
	if case:
		allfiles = SORT(String1d(os.listdir(directory)))
	else:
		allfiles = String1d([file.lower() for file in SORT(String1d(os.listdir(directory)))])
	files = allfiles
	if pattern:
		files = [file for file in allfiles if fnmatch.fnmatch(file, pattern)]
	if verbose: 
		for file in files: print file
	return files


