#Demonstration on how to simply obtain large aperture photometry of slightly extended sources
#on PACS level 2.5 maps from HSA
#
#Spectrometer Workshop: PACS
#23 April 2014
#Created by E. Puga

from herschel.share.fltdyn.time import *
from java.util import Date
from java.text import DateFormat
from java.text import SimpleDateFormat
from herschel.share.fltdyn.time import FineTime
from herschel.share.unit import Angle
from herschel.share.unit import Angle
from java.util import Calendar
from math import *

df=SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ")
j2000="2000-01-01T00:00:00Z"
date2000=FineTime(df.parse(j2000.split("Z")[0]+"-0000"))
secTorad=Angle.SECONDS_ARC.to(Angle.RADIANS)
degTorad=Angle.DEGREES.to(Angle.RADIANS)

def aperPhot(image, RA, Dec, skySizeX, skySizeY, skyPA, distBack, rad):
  #distBack is the factor used to multiply the diameter of the target aperture to obtain the position of the sky aperture  
  ss = image.getDimensions()
  wcs = image.getWcs()
  tgtYPix, tgtXPix = wcs.getPixelCoordinates(RA,Dec)
  cdelt1=ABS(wcs.cdelt1)*3600.
  cdelt2=wcs.cdelt2*3600.
  skySizeXPix = skySizeX/cdelt1
  skySizeYPix = skySizeY/cdelt2
  xsin=SIN(skyPA/180.*Math.PI)
  ycos=COS(skyPA/180.*Math.PI)
  separation = distBack*2*rad
  skyMinXPix = tgtXPix-xsin*separation/cdelt1-skySizeXPix/2.
  skyMinYPix = tgtYPix+ycos*separation/cdelt2-skySizeYPix/2.  
  print skyMinXPix, skyMinYPix, skySizeXPix, skySizeYPix, rad/cdelt1
  res = rectangularSkyAperturePhotometry(image=image, centerX=tgtXPix, centerY=tgtYPix, 
                                         minX=skyMinXPix, minY=skyMinYPix, widthPixels=skySizeXPix, heightPixels=skySizeYPix, centroid=True, radiusPixels=rad/cdelt1, fractional=1)        
  #Display image and visualize apertures
  d=Display(image)
  d.setCutLevels(90.0)
  d.addCross(tgtYPix, tgtXPix, 5, 3, java.awt.Color.green) 
  d.addCircle(tgtYPix, tgtXPix, rad/cdelt1, 1, java.awt.Color.blue)
  d.addRectangle( skyMinYPix, skyMinXPix, skySizeXPix, skySizeYPix, 1, java.awt.Color.black)
  
  return res.targetTotal, res.targetError

band=Double1d([70.0, 100.0, 160.])
fwhm=Double1d([5.8, 6.9, 12.1])#in arcseconds

###################################################################
#source name, obsids and photometric parameters                 
###################################################################

source='NGC7027'
obsids={}
obsids['NGC7027']=[1342195839]
apPhot={}
apPhot['NGC7027']={'radius':100, 'distBack':1., 'skyPA':120.}


tabSource=String1d()
tabWave=Double1d()
tabRad=Double1d()
tabFlux=Double1d()
tabeFlux=Double1d()

for obsid in obsids[source]:
    obs=getObservation(obsid, useHsa=1)
    for camera in ['blue','red']:
       #Take the level2.5 photProject product
       image = obs.level2_5.mosaic.getCamera(camera).product.refs['photProject'].product
       # Coordinates and wcs
       ranom = obs.meta["raNominal"].value
       decnom = obs.meta["decNominal"].value
       # RA-DEC correction due to proper motion
       pmRa = obs.properMotion[0]
       pmDec = obs.properMotion[1]
       start = obs.startDate
       ellaps = start.subtract(date2000)/(365*24*3600e6)
       corr_ra = pmRa*ellaps/COS(degTorad*decnom)
       corr_dec = pmDec*ellaps
       ranom = ranom+corr_ra/3600.
       decnom = decnom+corr_dec/3600.
       ss = image.getDimensions()
       wcs = image.getWcs()
       cpos = wcs.getPixelCoordinates(ranom,decnom)
       cdelt1 = ABS(wcs.cdelt1)*3600.
       cdelt2 = wcs.cdelt2*3600.
       wave = image.meta['wavelength'].value
       idx = fwhm.where(band == wave)
       sizeX = 20*fwhm[idx][0]/cdelt1
       sizeY = 20*fwhm[idx][0]/cdelt2  
       print 'Postage stamp for Gaussian fit: %.1f\"x%.1f\" ' % (sizeX, sizeY)
       #find centroid fitting a Gaussian in a postage stamp of size 20*fwhm: I assume that the centroid is going to be very similar at all wavelengths
       param = sourceFitting(image=image, minX=CEIL(cpos[1]-sizeX/2.), minY=CEIL(cpos[0]-sizeY/2.), width=sizeX, height=sizeY, elongated=True, slope=True)
    
       flux, eflux = aperPhot(image, param.centerRA, param.centerDec, skySizeX=90., skySizeY=80., skyPA=apPhot[source]['skyPA'], distBack=apPhot[source]['distBack'], rad=apPhot[source]['radius'])
       print 'band: %.1f, Flux: %.3f +- %.3f Jy' %(wave, flux, eflux)

       tabSource.append(source)
       tabWave.append(wave)
       tabRad.append(apPhot[source]['radius'])
       tabFlux.append(flux)
       tabeFlux.append(eflux)

tab = TableDataset(description = "PACS photometry table")

tab["Source"] = Column(data=tabSource,
                       description="Name of Source")
tab["Band"] = Column(data=tabWave,
                       description="$\micro$m")
tab["Aperture.rad"] = Column(data=tabRad,
                       description="arcseconds")
tab["Flux"] = Column(data=tabFlux,
                       description="Flux in Jy")
tab["eFlux"] = Column(data=tabeFlux,
                       description="Flux error in Jy")

asciiTableWriter(table=tab, file='/home/epuga/PACS_ICC/SpecWorkshop/PACSPhoto.tbl')




