#######################################################################
#Preliminary concept demonstration on how to simulate PACS-S observations
#from a sky surface brightness model
#This is a basic concept test. No correction for response nor beam truncation is applied here
#
#The beam shape for each spaxel is obtained from the beam efficiencies present in the CalTree v3
#
#The PACS photometer map is used as a sky surface brightness model 
#
#1
#I overlay the beam efficiencies for each spaxel on the sky to make sure
#the rotation convention is correct (compare with PACS footprint viewer)
#2
#Rotate the map on the spacecraft coordinate system  and create wcs
#for the beams in this same reference system to let imageMultiply take care of the 
#projection on the beam fine grid. Calculate the integral per detector module 
#and place result in a 5x5 image.  
#
#For demo purposes, I only use the beams in R1 and wavelength 145 um
#
#Follow the interspersed comments for further details.
#######################################################################

#Have the footprint viewer as reference
#Photometer observation
obsidP=1342187078
obsP=getObservation(obsidP, useHsa=1)
mapR = obsP.refs["level2"].product.refs["HPPPMAPR"].product
footprintApi = pacsSpectralFootprint(input=mapR, plotLabels=True)

#Spectrometer observation
obsidS=1342222230
obsS=getObservation(obsidS, useHsa=1)
pacsRebinnedCube = obsS.refs["level2"].product.refs["HPS3DRR"].product.refs[0].product
pA=obsS.meta['posAngle'].value
ra=MEAN(pacsRebinnedCube.ra[:,2,2])
dec=MEAN(pacsRebinnedCube.dec[:,2,2])
footprintApi.addFootprint(pacsRebinnedCube)
wcs = mapR.wcs
row,col = wcs.getPixelCoordinates(ra,dec)
footprintApi.display.addCircle(row,col,0.25,0.1,java.awt.Color.GREEN)

calTree=getCalTree()

#Load the beam efficiencies for all 7 wavelengths in R1 band
band='R1'
beamsBand=calTree.refs["spectrometer"].product.refs["beamsPerSpaxel%s" % band].product
waves=beamsBand.wavelenghts #this is a typo in the methods wavelenghts and getWavelenghts
beamsBand_beams=beamsBand['beams'].data #so it is a Double3d and not an ArrayDataSet
print beamsBand_beams.dimensions
for wave in [150.0]:
    #selection of the beam products per wavelength
    idx=waves.where(waves == wave)
    posi=idx.toInt1d()[0]*25
    posf=posi+25
    subbeams=beamsBand_beams[posi:posf,:,:]
    #1
    # beams rotated on map
    d=Display(mapR)
    d.setCutLevels(100.0)
    for i in range(25):
      x = i%5
      y = i/5
      beamWcs = Wcs(crpix1 = 109, crpix2 = 109, crval1 = 0.0, crval2 = 0.0, \
      crpix1=54.5, crpix2=54.5, cdelt1 = -0.5/3600., cdelt2 = 0.5/3600., cunit1 = "DEGREES", \
      cunit2 = "DEGREES", ctype1 ="RA---TAN", ctype2 = "DEC--TAN", equinox=2000.0 )
      beam = SimpleImage(image=subbeams[i,:,:], wcs=beamWcs)
      angle=pA
      #rotate beams to the spectrometer's position angle
      bmr=beam.copy()
      bmr.wcs.setCrota2(-angle)
      bmr.setDescription("wcs rotated for "+str(obsidP)+", "+band)
      print "WCS of rotated beam:",bmr.wcs
      bmr.wcs.setCrval1(ra)
      bmr.wcs.setCrval2(dec)
      contoursbmr= automaticContour(image = bmr, levels = 3, min = 0.5, max = 1.2, \
               distribution = 1)
      d.addWcsImageContour(contoursbmr)
    #2
    #map rotated on beams
    mapR.wcs.setCrval1(0.0)
    mapR.wcs.setCrval2(0.0)
    mapR.wcs.setCrpix1(col)
    mapR.wcs.setCrpix2(row)
    mapR.wcs.setCrota2(angle)
    mapR.setDescription("wcs rotated for "+str(obsidS)+", "+band)
    #
    f=Double2d(5,5)
    for i in range(25):
       #Instead of module, we express the final image in spaxel Indices
       x = i%5
       y = i/5
       beamWcs = Wcs(crpix1 = 109, crpix2 = 109, crval1 = 0.0, crval2 = 0.0, \
       crpix1=54, crpix2=54, cdelt1 = -0.5/3600., cdelt2 = 0.5/3600., cunit1 = "DEGREES", \
       cunit2 = "DEGREES", ctype1 ="RA---TAN", ctype2 = "DEC--TAN", equinox=2000.0 )
       beam = SimpleImage(image=subbeams[i,:,:], wcs=beamWcs)
       #this is not strictly the solid angle, but the beam solid angle multiplied by the response. The response will cancel out with the final normalization
       beamArea=SUM(SUM(beam.image,0))
       #Pseudo 2d discrete convolution
       #ImageMultiply projects the photometry map in units of Jy/pixel (cdelt=6.4") onto the beam grid (cdelt=0.5")
       prod=imageMultiply(image1=beam, image2=mapR, ref=1) 
       #The conversion factor yields units of Jy/spaxel (cdelt=9.4")
       f[y,x]=SUM(SUM(prod.image,0))/beamArea*(9.4**2/0.5**2)
    flux = SimpleImage(image=f, wcs=bmr.wcs)
    Display(flux)


