#Demonstration on how to create an ad-hoc point-source correction for the PACS 5x5 integrated 
#SED spectrum of a semi-extended source (> 47"), using PACS large aperture photometry and a modified blackbody SED.
#
#Spectrometer Workshop: PACS
#23 April 2014
#Created by E. Puga


from math import *
from herschel.ia.gui.plot.renderer.PlotLegendEngine import Position

###################################################################
#functions
###################################################################


def flagSet(band,gd,flag,wve):
    flg=flag.copy()
    if (gd==1):
        if (band == "B3A"):
            sey=[56.0,70.0]
        elif (band == "B2A"):
            sey=[53.0,72.5]
        elif (band == "B2B"):
            sey=[71.0,98.0]
        else:
            sey=[100,190]
        idx=wve.where(wve < sey[0])
        flg[idx]= 1.0
        idx=wve.where(wve > sey[1])
        flg[idx]= 1.0
    else:
        sey=ABS(len(gd)/2.) - len(gd)/2
        if ((len(gd) > 0) &(sey != 0.0)):
            print "Error: you have an odd number of ""good"" entries"
            return
        flg[:]= 1.0
        for i in range(0,len(gd),2):
            idx=wve.where((wve >= gd[i])&(wve <= gd[i+1]))
            if (idx.length() == 0):
                print "Error: your limits",gd[i],gd[i+1],"are outside the range of the spectrum; start again"
                flg=None
            else: 
                flg[idx]= 0.0
    return(flg)

def colorShade(col=java.awt.Color(128,128,128), npts=9):
    r=col.getRed()
    g=col.getGreen()
    b=col.getBlue()
    nr=(npts-1)/2
    steps=Double1d.range(nr)*(1./(nr+1))+(1./(nr+1))
    listShades= [java.awt.Color(int(r*step), int(g*step), int(b*step)) for step in steps]
    listTints= [java.awt.Color(int(r+(step*(255-r))), int(g+(step*(255-g))), int(b+(step*(255-b)))) for step in steps]
    listAll=[listShades, [col], listTints]
    listFinal=[num for elem in listAll for num in elem]
    return listFinal

#A utility
def ls(directory="./",pattern="",case=1,verbose=1):
	"""ls(directory="./",pattern="",case=1)
Returns the files in "directory" (relative path allowed) with filename containing "pattern" (incl. wildcards)
case	: 0/1 for case (in)sensitive listing (prefer case = 1 for pattern filtering)
"""
	import sys,os,fnmatch
	if case:
		allfiles = SORT(String1d(os.listdir(directory)))
	else:
		allfiles = String1d([file.lower() for file in SORT(String1d(os.listdir(directory)))])
	files = allfiles
	if pattern:
		files = [file for file in allfiles if fnmatch.fnmatch(file, pattern)]
	if verbose: 
		for file in files: print file
	return files

###################################################################
#Data,  name and Model Parameters                 
###################################################################
path= '/home/epuga/PACS_ICC/SpecWorkshop/DATATBN/'
sourceName='Sh2-104IRS1'

obsids = {}
obsids[1342210815] = "SEDA" 
obsids[1342210816] = "SEDB"
#Each pool has been separated in red and blue

###################################################################

calTree=getCalTree()

#Load FinalCubes and concatenate them
cubes=[]
for obsid in obsids.keys():
   for camera in ['blue','red']:
      pattern='*'+ str(obsid) +'*'+ camera+'*'+ '_FinalCubes'
      files=ls(directory=path,pattern = pattern, case=1, verbose=0)
      for file in files:
         print file
         cubes.append(readSliced(file, poolLocation=path, verbose=0))
cubes=concatenateSliced(cubes)
#Plot of spectrum in central spaxel
x,y=2,2
pinit = plotCubes(cubes,[],x=x,y=y)

#Plot the 5x5 spaxels
p=plotCube5x5(cubes.get(0))
for i in range(1,cubes.getNumberOfSlices()):
  p=plotCube5x5(cubes.get(i), p)

#Sum of spectra in the 5x5 spaxels in PACS IFU
fullSpec = Spectrum1d()
segments = Int1d()
flags = Int1d()

for slice in range(len(cubes.refs)):
  rcube = cubes.get(slice)
  band = rcube.meta['band'].value
  c5 = extractRegionSpectrum(cube=rcube, arithmetics=herschel.ia.toolbox.cube.ExtractRegionSpectrumTask.Arithmetics.SUM)

  # Concatenate all bands (slices) to one spectrum1d, keeping them as separate segments
  spec = c5.spectrum1d #c1 is a SimpleSpectrum and we convert it to Spectrum1d
  fullSpec.concatenate(spec)
  segments.append(Int1d(spec.flux.length(), slice))
  flg = Int1d(spec.wave.length(), 0)
  flg=flagSet(band,1,flg,spec.wave)
  flags.append(flg)

# Include the columns segment and flag in the final Spectrum1d & plot
fullSpec.setSegment(segments)
fullSpec.setFlag(flags)

p=PlotXY()

npt=fullSpec.getSegmentCount()
colShade=colorShade(java.awt.Color(255,0,127),npt+1) 
for i in range(fullSpec.getSegmentCount()):
   try:
      wave,flux,flag = fullSpec.getSegment(i).wave, fullSpec.getSegment(i).flux, fullSpec.getSegment(i).flag
      idx= flag.where(flag == 0) 
      p.addLayer(LayerXY(wave[idx], flux[idx], line=1, color=colShade[i]))
   except:
      pass
p.legend.visible=1
p.legend.position = Position.RIGHTMIDDLE
p.titleText=sourceName
p.setXtitle("Wavelength ($\micro$m)")
p.setYtitle("Flux (Jy)")
p.xaxis.tick.gridLines = 1
p.yaxis.tick.gridLines = 1
p.xaxis.range=[50, 192]
p.setPlotSize(8, 3.0)
c = 299792458.0 # m/s
xaux = ReciprocalAuxAxis(1.0e-3*c)
p.getXaxis().removeAuxAxis(0)
p.getXaxis().addAuxAxis(xaux)
xaux.setTitleText("Frequency (GHz)")
p.xaxis.type=Axis.LOG
p.yaxis.type=Axis.LOG

###################################################################
#PHOTOMETRY
###################################################################

from herschel.share.fltdyn.time import *
from java.util import Date
from java.text import DateFormat
from java.text import SimpleDateFormat
from herschel.share.fltdyn.time import FineTime
from herschel.share.unit import Angle
from herschel.share.unit import Angle
from java.util import Calendar
from math import *

df=SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ")
j2000="2000-01-01T00:00:00Z"
date2000=FineTime(df.parse(j2000.split("Z")[0]+"-0000"))
secTorad=Angle.SECONDS_ARC.to(Angle.RADIANS)
degTorad=Angle.DEGREES.to(Angle.RADIANS)

def aperPhot(image, RA, Dec, skySizeX, skySizeY, skyPA, distBack, rad, cutLevel=90.0):
  #distBack is the factor used to multiply the diameter of the target aperture to obtain the position of the sky aperture  
  ss = image.getDimensions()
  wcs = image.getWcs()
  tgtYPix, tgtXPix = wcs.getPixelCoordinates(RA,Dec)
  cdelt1=ABS(wcs.cdelt1)*3600.
  cdelt2=wcs.cdelt2*3600.
  skySizeXPix = skySizeX/cdelt1
  skySizeYPix = skySizeY/cdelt2
  xsin=SIN(skyPA/180.*Math.PI)
  ycos=COS(skyPA/180.*Math.PI)
  separation = distBack*2*rad
  skyMinXPix = tgtXPix-xsin*separation/cdelt1-skySizeXPix/2.
  skyMinYPix = tgtYPix+ycos*separation/cdelt2-skySizeYPix/2.  
  print skyMinXPix, skyMinYPix, skySizeXPix, skySizeYPix, rad/cdelt1
  res = rectangularSkyAperturePhotometry(image=image, centerX=tgtXPix, centerY=tgtYPix, 
                                         minX=skyMinXPix, minY=skyMinYPix, widthPixels=skySizeXPix, heightPixels=skySizeYPix, centroid=True, radiusPixels=rad/cdelt1, fractional=1)        
  d=Display(image)
  d.setCutLevels(cutLevel)
  d.addCross(tgtYPix, tgtXPix, 5, 3, java.awt.Color.green) 
  d.addCircle(tgtYPix, tgtXPix, rad/cdelt1, 1, java.awt.Color.blue)
  #d.addRectangle( tgtYPix-sizeYPix/2., tgtXPix-sizeXPix/2., sizeXPix, sizeYPix, 1, java.awt.Color.green)
  d.addRectangle( skyMinYPix, skyMinXPix, skySizeXPix, skySizeYPix, 1, java.awt.Color.red)
  
  #There is no color correction (depends on the power law index)
  
  return res.targetTotal, res.targetError

band=Double1d([70.0, 100.0, 160.])
fwhm=Double1d([5.8, 6.9, 12.1])#in arcseconds


sourceP='Sh2-104IRS1'
obsidsP={}
obsidsP['Sh2-104IRS1']=[1342185575,1342219040] #Together with 1342185576
apPhot={}
apPhot['Sh2-104IRS1']={'radius':35., 'distBack':3., 'skyPA':120.}


tabSource=String1d()
tabWave=Double1d()
tabRad=Double1d()
tabFlux=Double1d()
tabeFlux=Double1d()

#Inspect the Spectrometer Footprint on the map
obsP=getObservation(obsidsP[sourceP][0], useHsa=1)
mapR = obsP.refs["level2"].product.refs["HPPPMAPR"].product
footprintApi = pacsSpectralFootprint(input=mapR, plotLabels=True)

#Spectrometer observation
obsS=getObservation(obsids.keys()[0], useHsa=1)
pacsRebinnedCube = obsS.refs["level2"].product.refs["HPS3DRR"].product.refs[0].product
footprintApi.addFootprint(pacsRebinnedCube)
footprintApi.display.setCutLevels(99.9)

for obsid in obsidsP[sourceP]:
    obs=getObservation(obsid, useHsa=1)
    for camera in ['blue','red']:
       #Take the level2.5 photProject product
       image = obs.level2_5.mosaic.getCamera(camera).product.refs['photProject'].product
       # Coordinates and wcs
       ra=MEAN(pacsRebinnedCube.ra[:,2,2])
       dec=MEAN(pacsRebinnedCube.dec[:,2,2])
       ranom = obs.meta["raNominal"].value
       decnom = obs.meta["decNominal"].value
       # RA-DEC correction due to proper motion
       pmRa = obs.properMotion[0]
       pmDec = obs.properMotion[1]
       start = obs.startDate
       ellaps = start.subtract(date2000)/(365*24*3600e6)
       corr_ra = pmRa*ellaps/COS(degTorad*decnom)
       corr_dec = pmDec*ellaps
       ra = ra+corr_ra/3600.
       dec = dec+corr_dec/3600.
       wv = image.meta['wavelength'].value
    
       flux, eflux = aperPhot(image, ra, dec, skySizeX=90., skySizeY=80., skyPA=apPhot[sourceP]['skyPA'], distBack=apPhot[sourceP]['distBack'], rad=apPhot[sourceP]['radius'], cutLevel=99.9)
       print 'band: %.1f, Flux: %.3f +- %.3f Jy' %(wv, flux, eflux)

       tabSource.append(sourceP)
       tabWave.append(wv)
       tabRad.append(apPhot[sourceP]['radius'])
       tabFlux.append(flux)
       tabeFlux.append(eflux)

tab = TableDataset(description = "PACS photometry table")

tab["Source"] = Column(data=tabSource,
                       description="Name of Source")
tab["Band"] = Column(data=tabWave,
                       description="$\micro$m")
tab["Aperture.rad"] = Column(data=tabRad,
                       description="arcseconds")
tab["Flux"] = Column(data=tabFlux,
                       description="Flux in Jy")
tab["eFlux"] = Column(data=tabeFlux,
                       description="Flux error in Jy")

asciiTableWriter(table=tab, file='/home/epuga/PACS_ICC/SpecWorkshop/PACSPhoto_S104.tbl')


###################################################################
#CORRECTION CALCULATION
###################################################################

def BlackBodyModified(x, par):
       # par[0] = Dust Mass (Msun)
       # par[1] = Temperature
       # par[2] = Beta
       c = 29979245800.0      # cm/s
       h = 6.6260755E-27    # erg s
       k = 1.380658E-16     # erg k-1
       d = 200.0 * 3.086E18 # distance in cm 
       kappa0 = 0.1
       nu0 = 1000.0         # GHz 1000 GHz = 300 mu
       nuG = c/(x*1E5)       # GHz
       nu = c/(x*1E-4)      # Hz
       Msol = 1.99E33       # g
       beta = par[2]
       temp = par[1]
       kappa =  kappa0 * (nuG/nu0)**beta
       return 1E23*par[0]*Msol*(1.0/d**2) * kappa * ((2*h*nu**3.0)/(c*c)) / ( EXP(h*nu/(k*temp)) - 1)


##### PHOTOMETRY FIT ######
sourceNamesP=tabSource
photWave=tabWave
photFlux=tabFlux
photeFlux=tabeFlux

#Add photometric points to the plot
idxp=sourceNamesP.where(sourceNamesP == sourceName)
p.addLayer(LayerXY(photWave[idxp], photFlux[idxp], errorY=[photeFlux[idxp], photeFlux[idxp]], line=0, symbol=Style.FCIRCLE, color=java.awt.Color.blue, name='PACS'))

model = MakePrivateNonLinearModel(BlackBodyModified,3)  ## set number of free (currently 3) parameters of the BBmod function (defined above)
model.setParameters(Double1d([ 1.0, 15.0, 2.0]))   ## initial guess for dust+gas mass, temp and beta.
fitter = LevenbergMarquardtFitter(photWave, model)
weights = (1./(photeFlux)**2)
paramsP = fitter.fit(photFlux, weights)
dustGassMass, temp, beta = paramsP[0],paramsP[1],paramsP[2]
print 'Photometric points'
print 'Dust+gass Mass= %.2f, Temperature= %.2f K, beta= %.2f' %(dustGassMass, temp, beta)

##### SPECTRUM FIT ######
specWave,specFlux,specFlag = fullSpec.wave, fullSpec.flux, fullSpec.flag
idx= specFlag.where(specFlag == 0) 

model = MakePrivateNonLinearModel(BlackBodyModified,3)  ## set number of free (currently 3) parameters of the BBmod function (defined above)
model.setParameters(Double1d([ 1.0, 15.0, 2.0]))   ## initial guess for dust+gas mass, temp and beta.
fitter = LevenbergMarquardtFitter(specWave[idx], model)
paramsS = fitter.fit(specFlux[idx])
dustGassMass, temp, beta = paramsS[0],paramsS[1],paramsS[2]
print 'Spectrum'
print 'Dust+gass Mass= %.2f, Temperature= %.2f K, beta= %.2f' %(dustGassMass, temp, beta)

##### Fits evaluation on wavelength grid to calculate correction ######
waveGrid=Double1d.range(int(CEIL((MAX(specWave)-MIN(specWave))/1.)))*1.+MIN(specWave)

fitPhot=BlackBodyModified(waveGrid,paramsP)
p.addLayer(LayerXY(waveGrid, fitPhot, line=Style.DASHED, color=java.awt.Color.gray, name='Phot MBB fit'))

fitSpec=BlackBodyModified(waveGrid,paramsS)
p.addLayer(LayerXY(waveGrid, fitSpec, line=Style.DASHED, color=java.awt.Color.darkGray, name='Spec MBB fit'))

corr25=fitPhot/fitSpec
p25=PlotXY()
p25.addLayer(LayerXY(waveGrid, fitPhot/fitSpec))
p25.setXtitle("Wavelength ($\micro$m)")
p25.setYtitle("Calculated correction")
p25.xaxis.tick.gridLines = 1
p25.yaxis.tick.gridLines = 1
p25.xaxis.range=[50, 192]
p25.yaxis.range=[0.7, 1.5]
p25.setPlotSize(8, 3.0)

##### Apply correction on Spectrum and display####################################

inter = LinearInterpolator(waveGrid, corr25, True)
npt=fullSpec.getSegmentCount()
colShade=colorShade(java.awt.Color.red, npt+1) 
for i in range(fullSpec.getSegmentCount()):
   try:
      wave,flux,flag = fullSpec.getSegment(i).wave, fullSpec.getSegment(i).flux, fullSpec.getSegment(i).flag
      correction = inter.of(wave)
      idx= flag.where(flag == 0) 
      p.addLayer(LayerXY(wave[idx], (flux*correction)[idx], line=1, color=colShade[i]))
   except:
      pass

